/** @file el2xxx_device.c Implementation of the device handler of the EL2xxx devices for the SOEM master. */

#include "stdincludes.h"
#include "soem_devices.h"

/* {{{ EL2004 */
#define EL2004_NCHAN            4
#define EL2004_DEV_ID           0x7d43052
#define EL2004_DEV_STR          "EL2004"

typedef struct PACKED
{
    uint8_t outbits;
} out_EL2004t;

void el2004_write(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL2004t *output_data = soem_di->output_address;

    /* Convert to int, and discard all but the lower EL2004_NCHAN bits. */
    output_data->outbits = ((int)val) & ((1 << EL2004_NCHAN) - 1);
}

static const OutputIoDefinition el2004_dout = {
    {DK_DOUT, "dout", EL2004_NCHAN}, &el2004_write
};

static const InputIoDefinition *el2004_inputs[] = {NULL};
static const OutputIoDefinition *el2004_outputs[] = {&el2004_dout, NULL};

const DeviceDefinition el2004_device = {
    EL2004_DEV_STR,
    el2004_inputs, el2004_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL2004_DEV_ID,
};
/* }}} */
/* {{{ EL2008 */
#define EL2008_NCHAN            8
#define EL2008_DEV_ID           0x7d83052
#define EL2008_DEV_STR          "EL2008"

typedef struct PACKED
{
    uint8_t outbits;
} out_EL2008t;

static void el2008_write(IoInstance *io_intance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_intance->device;
    out_EL2008t *output_device = soem_di->output_address;

    /* Convert to int, and discard all but the lower EL2008_NCHAN bits. */
    output_device->outbits = ((int)val) & ((1 << EL2008_NCHAN) - 1);
}

static const OutputIoDefinition el2008_dout = {
    {DK_DOUT, "dout", EL2008_NCHAN}, &el2008_write
};

static const InputIoDefinition *el2008_inputs[] = {NULL};
static const OutputIoDefinition *el2008_outputs[] = {&el2008_dout, NULL};

const DeviceDefinition el2008_device = {
    EL2008_DEV_STR,
    el2008_inputs, el2008_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL2008_DEV_ID,
};
/* }}} */
/* {{{ EL2809 */
#define EL2809_NCHAN            16
#define EL2809_DEV_ID           0x0AF93052
#define EL2809_DEV_STR          "EL2809"

typedef struct PACKED
{
    uint16_t outbits;
} out_EL2809t;

static void el2809_write(IoInstance *io_intance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_intance->device;
    out_EL2809t *output_device = soem_di->output_address;

    /* Convert to int, and discard all but the lower EL2809_NCHAN bits. */
    output_device->outbits = ((int)val) & ((1 << EL2809_NCHAN) - 1);
}

static const OutputIoDefinition el2809_dout = {
    {DK_DOUT, "dout", EL2809_NCHAN}, &el2809_write
};

static const InputIoDefinition *el2809_inputs[] = {NULL};
static const OutputIoDefinition *el2809_outputs[] = {&el2809_dout, NULL};

const DeviceDefinition el2809_device = {
    EL2809_DEV_STR,
    el2809_inputs, el2809_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL2809_DEV_ID,
};
/* }}} */
