/** @file el3xxx_device.c Implementation of the device handler of the EL3xxx devices for the SOEM master. */

#include "stdincludes.h"
#include "soem_devices.h"

/* {{{ EL3102 */
#define EL3102_RESOLUTION       (20.0/65535.0)
#define EL3102_RANGE            20.0
#define EL3102_NCHAN            2
#define EL3102_DEV_ID           0x0c1e3052
#define EL3102_DEV_STR          "EL3102"

typedef struct PACKED
{
    uint8_t status1;
    int16_t invalue1;
    uint8_t status2;
    int16_t invalue2;
} in_EL3102t;

static double el3102_read_ain0(IoInstance *io_instance) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    const in_EL3102t *input_device = soem_di->input_address;

    return input_device->invalue1 * EL3102_RESOLUTION;
}

static double el3102_read_ain1(IoInstance *io_instance) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    const in_EL3102t *input_device = soem_di->input_address;

    return input_device->invalue2 * EL3102_RESOLUTION;
}

static const InputIoDefinition el3102_ain0 = {
    {DK_AIN, "ain0", 1}, &el3102_read_ain0
};
static const InputIoDefinition el3102_ain1 = {
    {DK_AIN, "ain1", 1}, &el3102_read_ain1
};

static const InputIoDefinition *el3102_inputs[] = {&el3102_ain0, &el3102_ain1, NULL};
static const OutputIoDefinition *el3102_outputs[] = {NULL};

const DeviceDefinition el3102_device = {
    EL3102_DEV_STR,
    el3102_inputs, el3102_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL3102_DEV_ID,
};

/* }}} */
/* {{{ EL3104 */
#define EL3104_RESOLUTION       (20.0/65535.0)
#define EL3104_RANGE            20.0
#define EL3104_NCHAN            4
#define EL3104_DEV_ID           0x0c203052
#define EL3104_DEV_STR          "EL3104"

typedef struct PACKED
{
    uint16_t status1;
    int16_t  invalue1;
    uint16_t status2;
    int16_t  invalue2;
    uint16_t status3;
    int16_t  invalue3;
    uint16_t status4;
    int16_t  invalue4;
} in_EL3104t;

static double el3104_read_ain0(IoInstance *io_instance) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    const in_EL3104t *input_device = soem_di->input_address;

    return input_device->invalue1 * EL3104_RESOLUTION;
}

static double el3104_read_ain1(IoInstance *io_instance) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    const in_EL3104t *input_device = soem_di->input_address;

    return input_device->invalue2 * EL3104_RESOLUTION;
}

static double el3104_read_ain2(IoInstance *io_instance) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    const in_EL3104t *input_device = soem_di->input_address;

    return input_device->invalue3 * EL3104_RESOLUTION;
}

static double el3104_read_ain3(IoInstance *io_instance) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    const in_EL3104t *input_device = soem_di->input_address;

    return input_device->invalue4 * EL3104_RESOLUTION;
}

static const InputIoDefinition el3104_ain0 = {
    {DK_AIN, "ain0", 1}, &el3104_read_ain0
};
static const InputIoDefinition el3104_ain1 = {
    {DK_AIN, "ain1", 1}, &el3104_read_ain1
};
static const InputIoDefinition el3104_ain2 = {
    {DK_AIN, "ain2", 1}, &el3104_read_ain2
};
static const InputIoDefinition el3104_ain3 = {
    {DK_AIN, "ain3", 1}, &el3104_read_ain3
};

static const InputIoDefinition *el3104_inputs[] = {
    &el3104_ain0, &el3104_ain1, &el3104_ain2, &el3104_ain3, NULL
};

static const OutputIoDefinition *el3104_outputs[] = {NULL};

const DeviceDefinition el3104_device = {
    EL3104_DEV_STR,
    el3104_inputs, el3104_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL3104_DEV_ID,
};
/* }}} */
