/** @file el4xxx_device.c Implementation of the device handler of the EL4xxx devices for the SOEM master. */

#include "stdincludes.h"
#include "soem_devices.h"

/**
 * Convert real output value to value for the device, clamping against the limits.
 * @param val Value to convert.
 * @return Value to send to the port.
 */
static int16_t convert(double val) {
    float number = (val * 32767.0) / 10.0;
    if (number > 32767) return 32767;
    if (number < -32767) return -32767;
    return (int16_t)number;
}

/* {{{ EL4038 */
#define EL4038_DAC_RESOLUTION   (20.0/65535.0)
#define EL4038_DAC_RANGE        20.0
#define EL4038_NCHAN            8
#define EL4038_DEV_ID           0x0fc63052
#define EL4038_DEV_STR          "EL4038"

typedef struct PACKED
{
    int16_t outvalue1;
    int16_t outvalue2;
    int16_t outvalue3;
    int16_t outvalue4;
    int16_t outvalue5;
    int16_t outvalue6;
    int16_t outvalue7;
    int16_t outvalue8;
} out_EL4038t;

static void el4038_write_aout0(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue1 = convert(val);
}
static void el4038_write_aout1(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue2 = convert(val);
}
static void el4038_write_aout2(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue3 = convert(val);
}
static void el4038_write_aout3(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue4 = convert(val);
}
static void el4038_write_aout4(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue5 = convert(val);
}
static void el4038_write_aout5(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue6 = convert(val);
}
static void el4038_write_aout6(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue7 = convert(val);
}
static void el4038_write_aout7(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4038t *device = soem_di->output_address;

    device->outvalue8 = convert(val);
}

static const OutputIoDefinition el4038_aout0 = {
    {DK_AOUT, "aout0", 1}, &el4038_write_aout0
};
static const OutputIoDefinition el4038_aout1 = {
    {DK_AOUT, "aout1", 1}, &el4038_write_aout1
};
static const OutputIoDefinition el4038_aout2 = {
    {DK_AOUT, "aout2", 1}, &el4038_write_aout2
};
static const OutputIoDefinition el4038_aout3 = {
    {DK_AOUT, "aout3", 1}, &el4038_write_aout3
};
static const OutputIoDefinition el4038_aout4 = {
    {DK_AOUT, "aout4", 1}, &el4038_write_aout4
};
static const OutputIoDefinition el4038_aout5 = {
    {DK_AOUT, "aout5", 1}, &el4038_write_aout5
};
static const OutputIoDefinition el4038_aout6 = {
    {DK_AOUT, "aout6", 1}, &el4038_write_aout6
};
static const OutputIoDefinition el4038_aout7 = {
    {DK_AOUT, "aout7", 1}, &el4038_write_aout7
};

static const InputIoDefinition *el4038_inputs[] = {NULL};
static const OutputIoDefinition *el4038_outputs[] = {
    &el4038_aout0, &el4038_aout1, &el4038_aout2, &el4038_aout3,
    &el4038_aout4, &el4038_aout5, &el4038_aout6, &el4038_aout7, NULL
};

const DeviceDefinition el4038_device = {
    EL4038_DEV_STR,
    el4038_inputs, el4038_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL4038_DEV_ID,
};
/* }}} */
/* {{{ EL4132 */
#define EL4132_DAC_RESOLUTION   (20.0/65535.0)
#define EL4132_DAC_RANGE        20.0
#define EL4132_NCHAN            2
#define EL4132_DEV_ID           0x10243052
#define EL4132_DEV_STR          "EL4132"

typedef struct PACKED
{
    int16_t outvalue1;
    int16_t outvalue2;
} out_EL4132t;

static void el4132_write_aout0(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4132t *device = soem_di->output_address;

    device->outvalue1 = convert(val);
}

static void el4132_write_aout1(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4132t *device = soem_di->output_address;

    device->outvalue2 = convert(val);
}

static const OutputIoDefinition el4132_aout0 = {
    {DK_AOUT, "aout0", 1}, &el4132_write_aout0
};
static const OutputIoDefinition el4132_aout1 = {
    {DK_AOUT, "aout1", 1}, &el4132_write_aout1
};

static const InputIoDefinition *el4132_inputs[] = {NULL};
static const OutputIoDefinition *el4132_outputs[] = {
    &el4132_aout0, &el4132_aout1, NULL,
};

const DeviceDefinition el4132_device = {
    EL4132_DEV_STR,
    el4132_inputs, el4132_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL4132_DEV_ID,
};
/* }}} */
/* {{{ EL4134 */
#define EL4134_DAC_RESOLUTION   (20.0/65535.0)
#define EL4134_DAC_RANGE        20.0
#define EL4134_NCHAN            4
#define EL4134_DEV_ID           0x10263052
#define EL4134_DEV_STR          "EL4134"

typedef struct PACKED
{
    int16_t outvalue1;
    int16_t outvalue2;
    int16_t outvalue3;
    int16_t outvalue4;
} out_EL4134t;

static void el4134_write_aout0(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4134t *device = soem_di->output_address;

    device->outvalue1 = convert(val);
}

static void el4134_write_aout1(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4134t *device = soem_di->output_address;

    device->outvalue2 = convert(val);
}

static void el4134_write_aout2(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4134t *device = soem_di->output_address;

    device->outvalue3 = convert(val);
}

static void el4134_write_aout3(IoInstance *io_instance, double val) {
    DeviceInstance *soem_di = (DeviceInstance *)io_instance->device;
    out_EL4134t *device = soem_di->output_address;

    device->outvalue4 = convert(val);
}

static const OutputIoDefinition el4134_aout0 = {
    {DK_AOUT, "aout0", 1}, &el4134_write_aout0
};
static const OutputIoDefinition el4134_aout1 = {
    {DK_AOUT, "aout1", 1}, &el4134_write_aout1
};
static const OutputIoDefinition el4134_aout2 = {
    {DK_AOUT, "aout2", 1}, &el4134_write_aout2
};
static const OutputIoDefinition el4134_aout3 = {
    {DK_AOUT, "aout3", 1}, &el4134_write_aout3
};

static const InputIoDefinition *el4134_inputs[] = {NULL};
static const OutputIoDefinition *el4134_outputs[] = {
    &el4134_aout0, &el4134_aout1, &el4134_aout2, &el4134_aout3, NULL,
};

const DeviceDefinition el4134_device = {
    EL4134_DEV_STR,
    el4134_inputs, el4134_outputs,
    sizeof(DeviceInstance),
    NULL,
    EL4134_DEV_ID,
};
/* }}} */
