/** @file filterblock.h Filter definition. */

#ifndef FILTER_H
#define FILTER_H

typedef struct MemoryStruct Memory;

/** Definition of a filter. */
struct FilterBlockStruct {
    double *weights;      /**< Multiplication weights of each sample. */
    double *samples;      /**< Round-robin buffer of the last #num_weights samples. */
    size_t num_weights;   /**< Number of weights and samples. */
    size_t sample_offset; /**< Index in #samples pointing to the oldest sample. */
    BOOL operational;     /**< If set, the filter is operational */
                          /**< (It has received enough samples to produce meaningful output.) */
};
typedef struct FilterBlockStruct FilterBlock;

void FilterBlockInit(FilterBlock *f);
BOOL FilterBlockSetWeights(FilterBlock *f, Memory *mem, const double *weights, size_t num_weights);
BOOL FilterBlockPrecheck(FilterBlock *f);

BOOL FilterBlockAddSample(FilterBlock *f, double sample);
double FilterBlockComputeOutput(FilterBlock *f);
double FilterBlockAddSampleCompute(FilterBlock *f, double sample);

#endif
