import sys

# If enabled, 'info' lines are not printed.
SKIP_INFO = False


def unduplicate(lines):
    """
    Remove double line numbers.
    """
    result = []
    last = None
    for line in lines:
        if line == last:
            continue
        last = line
        result.append(last)

    return result

def report_error(lines, mesg, type='error', keep_running=False):
    """
    Format and report an error, warning, or information message.
    Unless L{keep_running} is C{True}, fatal errors are fatal.

    @param lines: Lines concerning the error message. Set to L{None} if not applicable.
    @param keep_running: If C{True}, do not abort execution on error.
    @return Whether a fatal error was produced.
    """
    # Format kind of error.
    if type == 'warning':
        type_text = 'warning'
        fatal = False
    elif type == 'info':
        # Rquested to skip printing these lines?
        if SKIP_INFO:
            return False

        type_text = 'info'
        fatal = False
    else:
        type_text = 'ERROR'
        fatal = True

    # Format lines, if present
    if lines is None or len(lines) == 0:
        lines_text = ""
    else:
        lines.sort()
        lines = unduplicate(lines)

        if len(lines) == 1:
            lines_text = " at line {}".format(lines[0])
        elif len(lines) == 2:
            lines_text = " at lines {} and {}".format(lines[0], lines[1])
        else:
            numbers = ", ".join(str(num) for num in lines[:-1])
            lines_text = " at lines {}, and {}".format(numbers, lines[-1])

    print("acqgen {}{}: {}".format(type_text, lines_text, mesg))
    if fatal and not keep_running:
        sys.exit(1)

    return fatal

